package kr.co.vividnext.sodalive.admin.content.curation

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationItem
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCurationItem.audioContentCurationItem
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AdminContentCurationItemRepository :
    JpaRepository<AudioContentCurationItem, Long>,
    AdminContentCurationItemQueryRepository

interface AdminContentCurationItemQueryRepository {
    fun findByCurationIdAndSeriesId(curationId: Long, seriesId: Long?): AudioContentCurationItem?
    fun findByCurationIdAndContentId(curationId: Long, contentId: Long?): AudioContentCurationItem?
    fun findByCurationIdAndItemId(curationId: Long, itemId: Long): AudioContentCurationItem?
    fun getAudioContentCurationItemList(curationId: Long): List<GetCurationItemResponse>
    fun getAudioContentCurationSeriesItemList(curationId: Long): List<GetCurationItemResponse>
}

class AdminContentCurationItemQueryRepositoryImpl(
    val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AdminContentCurationItemQueryRepository {
    override fun findByCurationIdAndSeriesId(curationId: Long, seriesId: Long?): AudioContentCurationItem? {
        return queryFactory
            .selectFrom(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContentCurationItem.series, series)
            .where(
                audioContentCurationItem.curation.id.eq(curationId),
                audioContentCurationItem.series.id.eq(seriesId)
            )
            .fetchFirst()
    }

    override fun findByCurationIdAndContentId(curationId: Long, contentId: Long?): AudioContentCurationItem? {
        return queryFactory
            .selectFrom(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContentCurationItem.content, audioContent)
            .where(
                audioContentCurationItem.curation.id.eq(curationId),
                audioContentCurationItem.content.id.eq(contentId)
            )
            .fetchFirst()
    }

    override fun findByCurationIdAndItemId(curationId: Long, itemId: Long): AudioContentCurationItem? {
        return queryFactory.selectFrom(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .where(audioContentCuration.id.eq(curationId), audioContentCurationItem.id.eq(itemId))
            .fetchFirst()
    }

    override fun getAudioContentCurationItemList(curationId: Long): List<GetCurationItemResponse> {
        return queryFactory
            .select(
                QGetCurationItemResponse(
                    audioContentCurationItem.id,
                    audioContent.title,
                    audioContent.detail,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.member.nickname.coalesce(""),
                    audioContent.isAdult
                )
            )
            .from(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContentCurationItem.content, audioContent)
            .where(
                audioContentCuration.id.eq(curationId),
                audioContentCurationItem.isActive.isTrue
            )
            .orderBy(audioContentCurationItem.orders.asc())
            .fetch()
    }

    override fun getAudioContentCurationSeriesItemList(curationId: Long): List<GetCurationItemResponse> {
        return queryFactory
            .select(
                QGetCurationItemResponse(
                    audioContentCurationItem.id,
                    series.title,
                    series.introduction,
                    series.coverImage.prepend("/").prepend(imageHost),
                    series.member.nickname.coalesce(""),
                    series.isAdult
                )
            )
            .from(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContentCurationItem.series, series)
            .where(
                audioContentCuration.id.eq(curationId),
                audioContentCurationItem.isActive.isTrue
            )
            .orderBy(audioContentCurationItem.orders.asc())
            .fetch()
    }
}
