package kr.co.vividnext.sodalive.live.room

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.room.cancel.CancelLiveRequest
import kr.co.vividnext.sodalive.live.room.donation.DeleteLiveRoomDonationMessage
import kr.co.vividnext.sodalive.live.room.donation.LiveRoomDonationRequest
import kr.co.vividnext.sodalive.live.room.like.LiveRoomLikeHeartRequest
import kr.co.vividnext.sodalive.live.room.visit.LiveRoomVisitService
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/live/room")
class LiveRoomController(
    private val service: LiveRoomService,
    private val visitService: LiveRoomVisitService
) {

    @GetMapping
    fun getRoomList(
        @RequestParam timezone: String,
        @RequestParam dateString: String? = null,
        @RequestParam status: LiveRoomStatus,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        ApiResponse.ok(
            service.getRoomList(
                dateString,
                status,
                isAdultContentVisible ?: true,
                pageable,
                member,
                timezone
            )
        )
    }

    @PostMapping
    fun createLiveRoom(
        @RequestPart("coverImage") coverImage: MultipartFile?,
        @RequestPart("request") requestString: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.createLiveRoom(coverImage, requestString, member))
    }

    @GetMapping("/detail/{id}")
    fun getRoomDetail(
        @PathVariable id: Long,
        @RequestParam timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRoomDetail(id, member, timezone))
    }

    @PostMapping("/enter")
    fun enterLive(
        @RequestBody request: EnterOrQuitLiveRoomRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.enterLive(request, member))
    }

    @PutMapping("/start")
    fun startLive(
        @RequestBody request: StartLiveRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.startLive(request, member))
    }

    @PutMapping("/cancel")
    fun cancelLive(
        @RequestBody request: CancelLiveRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.cancelLive(request, member))
    }

    @GetMapping("/recent-room-info")
    fun getRecentRoomInfo(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRecentRoomInfo(member))
    }

    @PutMapping("/{id}")
    fun editLiveRoomInfo(
        @PathVariable("id") roomId: Long,
        @RequestPart("coverImage") coverImage: MultipartFile?,
        @RequestPart("request") requestString: String?,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.editLiveRoomInfo(roomId, coverImage, requestString, member))
    }

    @GetMapping("/info/{id}")
    fun getRoomInfo(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRoomInfo(roomId = id, member))
    }

    @GetMapping("/donation-message")
    fun getDonationMessageList(
        @RequestParam roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getDonationMessageList(roomId, member))
    }

    @DeleteMapping("/donation-message")
    fun removeDonationMessage(
        @RequestBody request: DeleteLiveRoomDonationMessage,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.deleteDonationMessage(request, member))
    }

    @GetMapping("/{room_id}/profile/{user_id}")
    fun getUserProfile(
        @PathVariable("room_id") roomId: Long,
        @PathVariable("user_id") userId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getUserProfile(roomId, userId, member))
    }

    @GetMapping("/{id}/donation-total")
    fun donationTotal(
        @PathVariable("id") roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getDonationTotal(roomId, memberId = member.id!!))
    }

    @PutMapping("/info/set/speaker")
    fun setSpeaker(
        @RequestBody request: SetManagerOrSpeakerOrAudienceRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.setSpeaker(request))
    }

    @PutMapping("/info/set/listener")
    fun setListener(
        @RequestBody request: SetManagerOrSpeakerOrAudienceRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.setListener(request))
    }

    @PutMapping("/info/set/manager")
    fun setManager(
        @RequestBody request: SetManagerOrSpeakerOrAudienceRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.setManager(request, member))
    }

    @PostMapping("/donation")
    fun donation(
        @RequestBody request: LiveRoomDonationRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.donation(request, member))
    }

    @PostMapping("/donation/v2")
    fun donationV2(
        @RequestBody request: LiveRoomDonationRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.donationV2(request, member))
    }

    @PostMapping("/donation/refund/{id}")
    fun refundDonation(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.refundDonation(id, member))
    }

    @GetMapping("/{id}/donation-list")
    fun donationList(
        @PathVariable("id") roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getDonationStatus(roomId, memberId = member.id!!))
    }

    @PostMapping("/quit")
    fun quitRoom(
        @RequestParam("id") roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.quitRoom(roomId, member))
    }

    @GetMapping("/recent_visit_room/users")
    fun recentVisitRoomUsers(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(visitService.getRecentVisitRoomUsers(member))
    }

    @PostMapping("/like-heart")
    fun likeHeart(
        @RequestBody request: LiveRoomLikeHeartRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.likeHeart(request, member))
    }

    @GetMapping("/{id}/heart-total")
    fun getTotalHeartCount(
        @PathVariable("id") roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getTotalHeartCount(roomId))
    }

    @GetMapping("/{id}/heart-list")
    fun heartList(
        @PathVariable("id") roomId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getHeartList(roomId))
    }
}
