package kr.co.vividnext.sodalive.point

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.QPointGrantLog.pointGrantLog
import kr.co.vividnext.sodalive.point.QPointRewardPolicy.pointRewardPolicy
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface PointGrantLogRepository : JpaRepository<PointGrantLog, Long>, PointGrantLogQueryRepository

interface PointGrantLogQueryRepository {
    fun countByMemberIdAndPolicyIdAndStartDate(
        memberId: Long,
        policyId: Long,
        startDate: LocalDateTime,
        orderId: Long? = null
    ): Int

    fun getPointRewardStatusByMemberId(memberId: Long, timezone: String): List<GetPointRewardStatusResponse>
}

class PointGrantLogQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : PointGrantLogQueryRepository {
    override fun countByMemberIdAndPolicyIdAndStartDate(
        memberId: Long,
        policyId: Long,
        startDate: LocalDateTime,
        orderId: Long?
    ): Int {
        var where = pointGrantLog.memberId.eq(memberId)
            .and(pointGrantLog.policyId.eq(policyId))
            .and(pointGrantLog.createdAt.goe(startDate))

        if (orderId != null) {
            where = where.and(pointGrantLog.orderId.eq(orderId))
        }

        return queryFactory
            .select(pointGrantLog.id)
            .from(pointGrantLog)
            .where(where)
            .fetch()
            .size
    }

    override fun getPointRewardStatusByMemberId(memberId: Long, timezone: String): List<GetPointRewardStatusResponse> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                pointGrantLog.createdAt,
                "UTC",
                timezone
            ),
            "%Y.%m.%d | %H:%i:%s"
        )

        return queryFactory
            .select(
                QGetPointRewardStatusResponse(
                    pointGrantLog.point.stringValue().concat(" 포인트"),
                    formattedDate,
                    pointRewardPolicy.title
                )
            )
            .from(pointGrantLog)
            .innerJoin(pointRewardPolicy).on(pointGrantLog.policyId.eq(pointRewardPolicy.id))
            .where(pointGrantLog.memberId.eq(memberId))
            .orderBy(pointGrantLog.id.desc())
            .fetch()
    }
}
