package kr.co.vividnext.sodalive.search

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.series.genre.QSeriesGenre.seriesGenre
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.hashtag.QAudioContentHashTag.audioContentHashTag
import kr.co.vividnext.sodalive.content.hashtag.QHashTag.hashTag
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.keyword.QSeriesKeyword.seriesKeyword
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import kr.co.vividnext.sodalive.member.tag.QCreatorTag.creatorTag
import kr.co.vividnext.sodalive.member.tag.QMemberCreatorTag.memberCreatorTag
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class SearchRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun searchCreatorTotalCount(
        keyword: String,
        memberId: Long
    ): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        return queryFactory
            .select(member.id)
            .from(member)
            .leftJoin(memberCreatorTag).on(memberCreatorTag.member.id.eq(member.id))
            .leftJoin(creatorTag).on(memberCreatorTag.tag.id.eq(creatorTag.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(
                member.isActive.isTrue
                    .and(member.role.eq(MemberRole.CREATOR))
                    .and(creatorTag.isActive.isTrue.or(creatorTag.id.isNull))
                    .and(
                        member.nickname.containsIgnoreCase(keyword)
                            .or(member.introduce.containsIgnoreCase(keyword))
                            .or(creatorTag.tag.containsIgnoreCase(keyword))
                    )
                    .and(blockMember.id.isNull)
            )
            .groupBy(member.id)
            .fetch()
            .size
    }

    fun searchCreatorList(
        keyword: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<SearchResponseItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        return queryFactory
            .select(
                QSearchResponseItem(
                    member.id,
                    member.profileImage.prepend("/").prepend(imageHost),
                    member.nickname,
                    member.nickname
                )
            )
            .from(member)
            .leftJoin(memberCreatorTag).on(memberCreatorTag.member.id.eq(member.id))
            .leftJoin(creatorTag).on(memberCreatorTag.tag.id.eq(creatorTag.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(
                member.isActive.isTrue
                    .and(member.role.eq(MemberRole.CREATOR))
                    .and(creatorTag.isActive.isTrue.or(creatorTag.id.isNull))
                    .and(
                        member.nickname.containsIgnoreCase(keyword)
                            .or(member.introduce.containsIgnoreCase(keyword))
                            .or(creatorTag.tag.containsIgnoreCase(keyword))
                    )
                    .and(blockMember.id.isNull)
            )
            .groupBy(member.id)
            .orderBy(member.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    fun searchContentTotalCount(
        keyword: String,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContent.member.isActive.isTrue
            .and(audioContent.member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.limited.isNull)
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.title.containsIgnoreCase(keyword)
                    .or(audioContent.detail.containsIgnoreCase(keyword))
                    .or(audioContent.theme.theme.containsIgnoreCase(keyword))
                    .or(hashTag.tag.containsIgnoreCase(keyword).and(audioContentHashTag.isActive.isTrue))

            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(audioContentTheme).on(audioContentTheme.id.eq(audioContent.theme.id))
            .leftJoin(audioContentHashTag).on(audioContentHashTag.audioContent.id.eq(audioContent.id))
            .leftJoin(hashTag).on(audioContentHashTag.hashTag.id.eq(hashTag.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(audioContent.id)
            .fetch()
            .size
    }

    fun searchContentList(
        keyword: String,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<SearchResponseItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContent.member.isActive.isTrue
            .and(audioContent.member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.limited.isNull)
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.title.containsIgnoreCase(keyword)
                    .or(audioContent.detail.containsIgnoreCase(keyword))
                    .or(audioContent.theme.theme.containsIgnoreCase(keyword))
                    .or(hashTag.tag.containsIgnoreCase(keyword).and(audioContentHashTag.isActive.isTrue))

            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QSearchResponseItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    audioContent.member.nickname
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(audioContentTheme).on(audioContentTheme.id.eq(audioContent.theme.id))
            .leftJoin(audioContentHashTag).on(audioContentHashTag.audioContent.id.eq(audioContent.id))
            .leftJoin(hashTag).on(audioContentHashTag.hashTag.id.eq(hashTag.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(audioContent.id)
            .orderBy(audioContent.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    fun searchSeriesTotalCount(
        keyword: String,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(
                series.title.containsIgnoreCase(keyword)
                    .or(series.introduction.containsIgnoreCase(keyword))
                    .or(seriesGenre.genre.containsIgnoreCase(keyword))
                    .or(hashTag.tag.containsIgnoreCase(keyword).and(seriesKeyword.isActive.isTrue))
            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where
                .and(series.isAdult.isFalse)
                .and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(series.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(seriesGenre).on(seriesGenre.id.eq(series.genre.id))
            .leftJoin(seriesKeyword).on(seriesKeyword.series.id.eq(series.id))
            .leftJoin(hashTag).on(hashTag.id.eq(seriesKeyword.keyword.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id, series.orders)
            .fetch()
            .size
    }

    fun searchSeriesList(
        keyword: String,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<SearchResponseItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(
                series.title.containsIgnoreCase(keyword)
                    .or(series.introduction.containsIgnoreCase(keyword))
                    .or(seriesGenre.genre.containsIgnoreCase(keyword))
                    .or(hashTag.tag.containsIgnoreCase(keyword).and(seriesKeyword.isActive.isTrue))
            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where
                .and(series.isAdult.isFalse)
                .and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QSearchResponseItem(
                    series.id,
                    series.coverImage.prepend("/").prepend(imageHost),
                    series.title,
                    series.member.nickname
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(seriesGenre).on(seriesGenre.id.eq(series.genre.id))
            .leftJoin(seriesKeyword).on(seriesKeyword.series.id.eq(series.id))
            .leftJoin(hashTag).on(hashTag.id.eq(seriesKeyword.keyword.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id, series.orders)
            .orderBy(series.orders.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }
}
